@@ -49,7 +49,7 @@ class TestClusterObject(base.TestCase):
         cluster.cluster_template_id = wtypes.Unset
         self.assertEqual(wtypes.Unset, cluster.cluster_template_id)
 
-        # test backwards compatibility of bay fields with new objects
+        # test backwards compatibility of cluster fields with new objects
         cluster_dict['create_timeout'] = 15
         cluster = api_cluster.Cluster(**cluster_dict)
         self.assertEqual(15, cluster.create_timeout)
@@ -494,7 +494,9 @@ class TestPatch(api_base.FunctionalTest):
             '/clusters/%s/?rollback=True' % self.cluster_obj.uuid,
             [{'path': '/node_count', 'value': node_count,
               'op': 'replace'}],
-            headers={'OpenStack-API-Version': 'container-infra 1.3'})
+            headers={'OpenStack-API-Version': 'container-infra 1.3',
+                     "X-Roles": "member"
+                     })
 
         self.mock_cluster_update.assert_called_once_with(
             mock.ANY, node_count, self.cluster_obj.health_status,
@@ -507,7 +509,9 @@ class TestPatch(api_base.FunctionalTest):
             '/clusters/%s/?rollback=False' % self.cluster_obj.uuid,
             [{'path': '/node_count', 'value': node_count,
               'op': 'replace'}],
-            headers={'OpenStack-API-Version': 'container-infra 1.3'})
+            headers={'OpenStack-API-Version': 'container-infra 1.3',
+                     "X-Roles": "member"
+                     })
 
         self.mock_cluster_update.assert_called_once_with(
             mock.ANY, node_count, self.cluster_obj.health_status,
@@ -520,7 +524,9 @@ class TestPatch(api_base.FunctionalTest):
             '/clusters/%s' % self.cluster_obj.uuid,
             [{'path': '/node_count', 'value': node_count,
               'op': 'replace'}],
-            headers={'OpenStack-API-Version': 'container-infra 1.9'},
+            headers={'OpenStack-API-Version': 'container-infra 1.9',
+                     "X-Roles": "member"
+                     },
             expect_errors=True)
 
         self.assertEqual(400, response.status_code)
@@ -531,7 +537,9 @@ class TestPatch(api_base.FunctionalTest):
             '/clusters/%s' % self.cluster_obj.uuid,
             [{'path': '/node_count', 'value': node_count,
               'op': 'replace'}],
-            headers={'OpenStack-API-Version': 'container-infra 1.10'})
+            headers={'OpenStack-API-Version': 'container-infra 1.10',
+                     "X-Roles": "member"
+                     })
 
         self.mock_cluster_update.assert_called_once_with(
             mock.ANY, node_count, self.cluster_obj.health_status,
@@ -698,28 +706,34 @@ class TestPost(api_base.FunctionalTest):
         self.assertEqual(400, response.status_int)
         self.assertTrue(response.json['errors'])
 
-    def test_create_cluster_with_cluster_template_name(self):
+    def test_create_cluster_with_cluster_template_name_as_id(self):
         modelname = self.cluster_template.name
         bdict = apiutils.cluster_post_data(cluster_template_id=modelname)
-        response = self.post_json('/clusters', bdict, expect_errors=True)
+        response = self.post_json('/clusters', bdict)
         self.assertEqual('application/json', response.content_type)
         self.assertEqual(202, response.status_int)
 
     def test_create_cluster_with_zero_node_count_fail(self):
         bdict = apiutils.cluster_post_data()
         bdict['node_count'] = 0
-        response = self.post_json('/clusters', bdict, expect_errors=True,
-                                  headers={"Openstack-Api-Version":
-                                           "container-infra 1.9"})
+        response = self.post_json(
+            '/clusters', bdict, expect_errors=True,
+            headers={
+                "Openstack-Api-Version": "container-infra 1.9",
+                "X-Roles": "member"
+            })
         self.assertEqual('application/json', response.content_type)
         self.assertEqual(400, response.status_int)
 
     def test_create_cluster_with_zero_node_count(self):
         bdict = apiutils.cluster_post_data()
         bdict['node_count'] = 0
-        response = self.post_json('/clusters', bdict,
-                                  headers={"Openstack-Api-Version":
-                                           "container-infra 1.10"})
+        response = self.post_json(
+            '/clusters', bdict,
+            headers={
+                "Openstack-Api-Version": "container-infra 1.10",
+                "X-Roles": "member"
+            })
         self.assertEqual('application/json', response.content_type)
         self.assertEqual(202, response.status_int)
 
@@ -753,6 +767,40 @@ class TestPost(api_base.FunctionalTest):
         self.assertEqual('application/json', response.content_type)
         self.assertEqual(202, response.status_int)
 
+    def test_create_cluster_with_even_master_count_oldmicroversion(self):
+        bdict = apiutils.cluster_post_data()
+        bdict['master_count'] = 2
+        response = self.post_json(
+            '/clusters',
+            bdict,
+            expect_errors=True,
+            headers={"Openstack-Api-Version": "container-infra 1.9"}
+        )
+        self.assertEqual('application/json', response.content_type)
+        self.assertEqual(400, response.status_int)
+        self.assertTrue(response.json['errors'])
+
+    def test_create_cluster_with_even_master_count(self):
+        bdict = apiutils.cluster_post_data()
+        bdict['master_count'] = 2
+        response = self.post_json(
+            '/clusters',
+            bdict,
+            expect_errors=True,
+            headers={"Openstack-Api-Version": "container-infra 1.10"}
+        )
+        self.assertEqual('application/json', response.content_type)
+        self.assertEqual(400, response.status_int)
+        self.assertTrue(response.json['errors'])
+
+    def test_create_cluster_with_negative_master_count(self):
+        bdict = apiutils.cluster_post_data()
+        bdict['master_count'] = -1
+        response = self.post_json('/clusters', bdict, expect_errors=True)
+        self.assertEqual('application/json', response.content_type)
+        self.assertEqual(400, response.status_int)
+        self.assertTrue(response.json['errors'])
+
     def test_create_cluster_with_invalid_name(self):
         invalid_names = ['x' * 243, '123456', '123456test_cluster',
                          '-test_cluster', '.test_cluster', '_test_cluster', '']
@@ -860,7 +908,7 @@ class TestPost(api_base.FunctionalTest):
         self.assertTrue(self.mock_valid_os_res.called)
         self.assertEqual(409, response.status_int)
 
-    def test_create_cluster_with_on_os_distro_image(self):
+    def test_create_cluster_with_no_os_distro_image(self):
         bdict = apiutils.cluster_post_data()
         self.mock_valid_os_res.side_effect = \
             exception.OSDistroFieldNotFound('img')
